/* ###
 * IP: GHIDRA
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ghidra.app.plugin.core.overview;

import javax.swing.JComponent;

import docking.DialogComponentProvider;
import ghidra.util.HelpLocation;

/**
 * Convenience dialog for showing a Legend for an {@link OverviewColorComponent}
 */
public class OverviewColorLegendDialog extends DialogComponentProvider {

	/**
	 * Constructs a new {@link DialogComponentProvider} to show the given component with a "Dismiss"
	 * button.
	 *
	 * @param title the title of the dialog
	 * @param component the component to show as the main area of the dialog.
	 * @param help the help location for the dialog.
	 */
	public OverviewColorLegendDialog(String title, JComponent component, HelpLocation help) {
		super(title, false);
		component.getAccessibleContext().setAccessibleName("Overview Color Legend");
		addWorkPanel(component);
		addDismissButton();
		setHelpLocation(help);
		setRememberLocation(true);
		setRememberSize(false);
	}

	public void refresh() {
		getComponent().repaint();
	}

}
